/* print.c
 * Routines for printing packet analysis trees.
 *
 * Gilbert Ramirez <gram@verdict.uthscsa.edu>
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@zing.org>
 * Copyright 1998 Gerald Combs
 *
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include "config.h"

#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include "packet.h"
#include "print.h"

static
void dumpit (FILE *fh, register const u_char *cp, register u_int length);

/* Print a tree's data in plain text */
void print_tree_text(FILE *fh, const u_char *pd, GtkTree *tree)
{
	GList		*children, *child, *widgets, *label;
	GtkWidget	*subtree;
	int			num_children, i, j;
	char		*text;
	int			num_spaces;
	char		space[41];
	gint		data_start, data_len;

	/* Prepare the tabs for printing, depending on tree level */
	num_spaces = tree->level * 4;
	if (num_spaces > 40) {
		num_spaces = 40;
	}
	for (i = 0; i < num_spaces; i++) {
		space[i] = ' ';
	}
	/* The string is NUL-terminated */
	space[num_spaces] = 0;

	/* Get the children of this tree */
	children = tree->children;
	num_children = g_list_length(children);

	for (i = 0; i < num_children; i++) {
		/* Each child of the tree is a widget container */
		child = g_list_nth(children, i);
		widgets = gtk_container_children(GTK_CONTAINER(child->data));

		/* And the container holds a label object, which holds text */
		label = g_list_nth(widgets, 0);
		gtk_label_get(GTK_LABEL(label->data), &text);

		/* Print the text */
		fprintf(fh, "%s%s\n", space, text);

		/* Recurse into the subtree, if it exists */
		subtree = (GTK_TREE_ITEM(child->data))->subtree;
		if (subtree) {
				print_tree_text(fh, pd, GTK_TREE(subtree));
		}
		else if (strcmp("Data", text) == 0) {
			decode_start_len(GTK_TREE_ITEM(child->data), &data_start, &data_len);
			dumpit(fh, &pd[data_start], data_len);
		}
	}
}

/* This routine was created by Dan Lasley <DLASLEY@PROMUS.com>, and
only slightly modified for ethereal by Gilbert Ramirez. */
static
void dumpit (FILE *fh, register const u_char *cp, register u_int length)
{
        register int ad, i, j, k;
        u_char c;
        u_char line[60];
		static u_char binhex[16] = {
			'0', '1', '2', '3', '4', '5', '6', '7',
			'8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};

        memset (line, ' ', sizeof line);
        line[sizeof (line)-1] = 0;
        for (ad=i=j=k=0; i<length; i++) {
                c = *cp++;
                line[j++] = binhex[c>>4];
                line[j++] = binhex[c&0xf];
                if (i&1) j++;
                line[42+k++] = c >= ' ' && c < 0x7f ? c : '.';
                if ((i & 15) == 15) {
                        fprintf (fh, "\n%4x  %s", ad, line);
                        /*if (i==15) printf (" %d", length);*/
                        memset (line, ' ', sizeof line);
                        line[sizeof (line)-1] = j = k = 0;
                        ad += 16;
                }
        }

        if (line[0] != ' ') fprintf (fh, "\n%4x  %s", ad, line);
        fprintf(fh, "\n");
        return;

}


